'''
This script has no scientific purpose. It is just to visualize bubble growth by producing a composite image of the boundary as it moves throughout the run.

The 'unfolded' feature unfolds the bubble images, plotting r against phi for every frame on the same set of axes. The 'round' feature just overlays the binarized images on top of one another. I do it in matplotlib with the axes removed, and put it all on a black background (though that could of course be very easily changed).

Written by: Matthew Gerry
March, 2019
'''
from os import listdir
import sys
sys.path.append('FVF_code') # add to sys.path whatever directory contains FVF-related codes
from PIL import Image
from numpy import pi, sqrt, transpose, asmatrix, arctan
from matplotlib import pyplot as plt
from kill_outliers import kill_outliers
from operator import itemgetter

pi_by_2 = pi/2
plt.style.use('dark_background')
  
# choose whether you want to make a round or unfolded image
unfolded = False
round = True

path_to_image_folder = 'Desktop/skel/done'   #  a string specifying the path to the folder that will contain the input images
                            # folder and all the new folders with the processed images.

image_folder_name = '/3psi_D_2_empty_pipe_skel'  # the name of the folder with the input images.
    
skip = 4 # plot curves from only a fraction of the frames to make the image less crowded
    
x_centre = 940 # run fond_centre code beforehand to fix these parameters
y_centre = 838

file_list = listdir(path_to_image_folder+image_folder_name)  
file_list = sorted([name for name in file_list if (name.endswith('.tif') and not name.startswith('.'))]) 

# produce unfolded composite image
if unfolded == True:
    iframe = 0 # begin analyzing images from frame 0
    for image_file_name in file_list:
        if iframe%skip==0:
            print('start plotting image  '+str(iframe)+ '  ' + image_file_name)
        
            img = Image.open(path_to_image_folder+image_folder_name+'/'+image_file_name)  # create PIL image object
        
            img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case
        
            img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                                # the transpose makes the coordinates of images
                                                                # match those of matrices
            img.close()
            img_greyscale.close()
            
            radius = []
            angle = []
            
            for x in range(0, img_matrix.shape[0]): # search image, identifying radial and angular coordinates for the black pixels
                for y in range(0, img_matrix.shape[1]):
            
                    if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)
            
                        x_pos = float(x-x_centre)
                        y_pos = float(y-y_centre)
                    
                        radius = radius + [sqrt(x_pos**2 + y_pos**2)]  # append
            
                        if radius != 0.0:   # avoid zero radii, just in case 
            
                            if x_pos != 0.0:  # avoid a divide by zero situation
                                if x_pos > 0.0:
                                    angle = angle + [arctan(y_pos/x_pos)] # in radians
                                else:
                                    if y_pos > 0.0:
                                        angle = angle + [arctan(y_pos/x_pos) + pi]
                                    else:
                                        angle = angle + [arctan(y_pos/x_pos) - pi] #add/subtract pi as needed to correctly assign angle from arctan function
                            else:
                                if y_pos > 0.0:
                                    angle = angle + [pi_by_2]
                                else:
                                    angle = angle + [-pi_by_2]
            
                        else:  # angle is undefined for zero radius: just make it zero
                            angle = angle + [0.0]   # this should never happen
            
            for i in range(len(angle)):
                if angle[i]<0:
                    angle[i] = angle[i]+2*pi # move the gap caused by removing the pipe away from the centre of the data set
            # end of the loop over 1 frame
            
            # uses an interquartile radius test to eliminate outliers due to stray black pixels
            radius, angle = kill_outliers((radius,angle))
            
            # Put the pixels in order of increasing phi
            pairs = list(zip(angle, radius))
            pairs = sorted(pairs, key=itemgetter(0))
            sort_angle, sort_radius = [], []
            for i in range(len(pairs)):
                sort_angle = sort_angle + [pairs[i][0]]
                sort_radius = sort_radius + [pairs[i][1]]
            
            plt.plot(sort_angle, sort_radius, ',', color='white') #plot parameters
            plt.axis('off') #remove axes
            # Do NOT clear the plot after each iteration of the for loop
    
        
        iframe+=1 # move on to the next frame parameter forward
    
    plt.show() #show the plot just after the last frame has been plotted, when all the curves are present



if round == True: 
    iframe = 0 # begin analyzing images from frame 0
    for image_file_name in file_list:
        if iframe%skip==0:
            print('start plotting image  '+str(iframe)+ '  ' + image_file_name)
        
            img = Image.open(path_to_image_folder+image_folder_name+'/'+image_file_name)  # create PIL image object
        
            img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case
        
            img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                                # the transpose makes the coordinates of images
                                                                # match those of matrices
            img.close()
            img_greyscale.close()
    
            x_list = []
            y_list = []
            
            for x in range(0, img_matrix.shape[0]):
                for y in range(0, img_matrix.shape[1]):
            
                    if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)
            
                        x_pos = float(x) # simply produced lists of x-, y-coords of black pixels to plot
                        y_pos = float(y)
                    
                        x_list = x_list + [x_pos]
                        y_list = y_list + [y_pos]
                    
            # end of the loop over 1 frame
            
            plt.plot(x_list, y_list, ',', color='white') # plot parameters
            plt.gca().set_aspect('equal', adjustable='box')
            plt.draw()

            plt.xlim(0,img_matrix.shape[0]) # make the plot the size of the image
            plt.ylim(0,img_matrix.shape[1]) 
            plt.axis('off') #remove axes
        
        iframe+=1 # move on to the next frame parameter forward
    
    plt.show() #show the plot just after the last frame has been plotted, when all the curves are present
